 SETLI 132
;**************************************************;
;           S D D R I V E _ V I D E O              ; 
;           (c) 2021 - Daniel Coulom               ;  
;           http://dcmoto.free.fr/                 ;
;           http://forum.system-cfg.com/           ;
;--------------------------------------------------;
; Ce code est distribue gratuitement dans l'espoir ;
; qu'il sera utile, mais sans aucune  garantie  et ;
; sans  engager  la  responsabilit  de  l'auteur. ;
; Vous  pouvez  l' utiliser,  le  modifier  et  le ;
; diffuser librement, en conservant cette  licence ;
; et les rfrences de l'auteur dans   toutes  les ;
; copies. L'exploitation commerciale est interdite.;
;**************************************************;

; Ce programme utilise le controleur et l'interface
; SDDRIVE pour afficher une video en streaming.
; Le chargement et le lancement du programme sont
; realises grace a un secteur de boot.
    
;******************************************************
;                Version 2021.04.03                
;******************************************************
; Historique
; 2021.04.03 ajout test version 20210212 et suivantes  
; 2018.07.14 adresse SD_LB0 calculee selon version 
; 2018.07.02 deplacement de la zone SD_LB0 en <$51 
; 2018.02.04 ajout du son
; 2018.02.03 debut du developpement

;------------------------------------------------------
; DEBUT DU PROGRAMME
;------------------------------------------------------
  ORG   $9900
  BRA   INIT           ; initialisations 

;------------------------------------------------------
; FONCTION D'ACCES A LA CARTE SD
;------------------------------------------------------
CMD18
  FCB   $52            ; read multiple block
  FDB   $0000          ; adresse bloc (poids fort)
  FDB   $0000          ; adresse bloc (poids faible)
  FCB   $FF            ; checksum non testee 
  FCB   $00            ; code retour attendu  
CMD12
  FCB   $4C            ; stop transmission
  FDB   $0000          ; dummy parameter
  FDB   $0000          ; dummy parameter
  FCB   $FF            ; checksum non testee 
  FCB   $00            ; code retour attendu  

;------------------------------------------------------
; MACRO DE LECTURE D'UN OCTET AVEC SDDRIVE
; Le registre A doit contenir $7F avant l'appel
; Valeur de l'octet dans le registre B en sortie
; Duree totale de la lecture = 48 cycles
;------------------------------------------------------
READ_BYTE MACRO
  CMPA  <$BF           ; lecture bit 7              (4)
  ROLB                 ; pousser dans B             (2)
  CMPA  <$BF           ; lecture bit 6              (4)
  ROLB                 ; pousser dans B             (2)
  CMPA  <$BF           ; lecture bit 5              (4)
  ROLB                 ; pousser dans B             (2)
  CMPA  <$BF           ; lecture bit 4              (4)
  ROLB                 ; pousser dans B             (2)
  CMPA  <$BF           ; lecture bit 3              (4)
  ROLB                 ; pousser dans B             (2)
  CMPA  <$BF           ; lecture bit 2              (4)
  ROLB                 ; pousser dans B             (2)
  CMPA  <$BF           ; lecture bit 1              (4)
  ROLB                 ; pousser dans B             (2)
  CMPA  <$BF           ; lecture bit 0              (4)
  ROLB                 ; pousser dans B             (2)
  ENDM

;------------------------------------------------------
; MACRO D'ENVOI DE 8 TOPS D'HORLOGE
; Le registre B doit contenir $7F avant l'appel
; Duree totale de la macro = 32 cycles
;------------------------------------------------------
SEND_CLOCK MACRO
  CMPB  <$BF           ; lecture bit 7              (4)
  CMPB  <$BF           ; lecture bit 6              (4)
  CMPB  <$BF           ; lecture bit 5              (4)
  CMPB  <$BF           ; lecture bit 4              (4)
  CMPB  <$BF           ; lecture bit 3              (4)
  CMPB  <$BF           ; lecture bit 2              (4)
  CMPB  <$BF           ; lecture bit 1              (4)
  CMPB  <$BF           ; lecture bit 0              (4)
  ENDM

;------------------------------------------------------
; DETECTION DU TYPE D'ORDINATEUR TO OU MO
;------------------------------------------------------
INIT
  ORCC  #$50           ; desactive les interruptions 
  LDA   >$FFF2         ; $FFF2: $F0(MO) et $70(TO)
  BPL   INIT3          ; TO detecte

;------------------------------------------------------
; Initialisations communes MO5-MO6 
;------------------------------------------------------
  LDU   #$A7C0         ; adresse selection banque video
  LDY   #$0000         ; adresse debut ecran
  LDB   #$07           ; octet couleur
  BSR   COLOR          ; initialisation couleurs
  LDA   >$FFF0         ; type d'ordinateur
  BNE   INIT2          ; MO6   

;------------------------------------------------------
; Initialisations MO5 
; Attention aux differences PIA systeme MO5 et MO6
;------------------------------------------------------
INIT1
  LDA   >$A7C0         ; port A du PIA systeme
  ANDA  #$E1           ; raz couleur cadre
  ORA   #$0E           ; couleur cadre 7 (blanc) 
  STA   >$A7C0         ; change couleur cadre
  BRA   INIT8          ; suite des initialisations

;------------------------------------------------------
; Initialisations MO6 
; Attention aux differences PIA systeme MO5 et MO6
;------------------------------------------------------
INIT2
  LDA   #$07           ; couleur cadre 7 (blanc) 
  STA   >$A7DD         ; change couleur cadre
  LDA   >$A7C0         ; PIA systeme 
  ANDA  #$FB           ; clear mute bit 
  STA   >$A7C0         ; modification PIA
  BRA   INIT8          ; suite des initialisations

;------------------------------------------------------
; Initialisations communes TO7-TO8 
;------------------------------------------------------
INIT3
  LDU   #$E7C3         ; adresse selection banque video
  LDY   #$5F40         ; adresse fin ecran
  STY   VIDEO3+2       ; initialisation adresse fin ecran
  LDY   #$4000         ; adresse debut ecran
  STY   VIDEO+1        ; initialisation adresse debut ecran
  LDB   #$C7           ; octet couleur
  BSR   COLOR          ; initialisation couleurs
  LDA   #$E0           ; valeur adresse EXCMD (poids fort)
  STA   READ+4         ; adresse execution EXCMD (pour CMD18)
  STA   FIN+4          ; adresse execution EXCMD (pour CMD12)
  LDA   >$FFF0         ; type d'ordinateur
  BEQ   INIT4          ; TO7=0   
  DECA                 ; retrancher 1
  BEQ   INIT4          ; TO7/70=1
  BRA   INIT5          ; TO8=3   

;------------------------------------------------------
; Initialisations TO7 
;------------------------------------------------------
INIT4
  LDA   >$E7C3         ; port A du PIA systeme
  ORA   #$70           ; couleur cadre 7 (blanc) 
  STA   >$E7C3         ; change couleur cadre
  BRA   INIT8          ; suite des initialisations

;------------------------------------------------------
; Initialisations TO8 
;------------------------------------------------------
INIT5
  LDA   #$07           ; couleur cadre 7 (blanc) 
  STA   >$E7DD         ; change couleur cadre TO8
  BRA   INIT8          ; suite des initialisations

;------------------------------------------------------
; Initialisation couleur noir/blanc (Y = debut ecran)
; suivie de l'initialisation forme a zero
; Adresse du bit de selection banque video dans U
; Adresse octet couleur dans B
;------------------------------------------------------
COLOR   
  LDA   ,U             ; octet de selection banque video
  ANDA  #$FE           ; clear bit 0
  STA   ,U             ; selection video couleur
  LDX   #$1F40         ; compteur de boucles
  TFR   B,A            ; code couleur sur deux octets
COLOR1  
  STD   ,Y++           ; initialise deux octets
  LEAX  -2,X           ; comptage
  BNE   COLOR1         ; boucle
  INC   ,U             ; selection video forme
  LDX   #$1F40         ; compteur de boucles
  CLRA                 ; code forme
  CLRB                 ; sur deux octets 
  LEAY  -$1F40,Y       ; adresse debut ecran   
FORME  
  STD   ,Y++           ; initialise deux octets
  LEAX  -2,X           ; comptage
  BNE   FORME          ; boucle
  RTS

;------------------------------------------------------
; Initialisations communes 
;------------------------------------------------------

; Initialisation adresse SD_LB0
; Avant juillet 2018, SD_LB0 etait en $208E/$608E
; Depuis juillet 2018, SD_LB0  est en $2051/$6051
; La nouvelle adresse est utilise dans deux cas :
; - $55 dans les bits 0-7 de <$58 (20180702-20210211)
; - $55 dans les bits 0-6 de <$57 (20210212 et suivantes) 

INIT8
  LDB   #$51           ; adresse de SD_LB0 depuis 20180702
  LDA   <$58           ; <$58 = $55 pour 20180702-20210211   
  CMPA  #$55           ; test $55
  BEQ   INIT9          ; version 20180702-20210211
  LDA   <$57           ; <$57 b0-b6 = $55 depuis 20210212   
  ANDA  #$7F           ; isole b0-b6 
  CMPA  #$55           ; test $55
  BEQ   INIT9          ; version 20210212 ou plus recente
  LDB   #$8E           ; adresse de SD_LB0 avant 20180702

; Initialisation adresse du fichier .sd pour CMD18
INIT9
  TFR   DP,A           ; valeur de DP dans A
  TFR   D,X            ; adresse SD_LB0 stockee dans X
  LDD   ,X             ; octets poids fort adresse
  STD   CMD18+1        ; stockage pour commande CMD18
  LDD   2,X            ; octets poids faible adresse
  STD   CMD18+3        ; stockage pour commande CMD18

; Initialisation CNA en sortie
  TFR   DP,B           ; valeur actuelle de DP
  ADDB  #$87           ; $A7 pour MO et $E7 pour TO
  TFR   B,DP           ; initialisation DP
  LDA   <$CF           ; lecture registre de controle B
  ANDA  #$FB           ; raz bit 2 
  STA   <$CF           ; selection DDRB
  LDB   #$3F           ; set bits 0-5 
  STB   <$CD           ; bits CNA en sortie
  ORA   #$04           ; set b2
  STA   <$CF           ; selection PB

;-------------------------------------------------------
; Lancement de la commande CMD18 et saut de huit blocs
;-------------------------------------------------------
READ
  LDU   #CMD18         ; adresse commande CMD18
  JSR   $A028          ; EXCMD = execution commande
;                        READ+4 modifie en $E028 pour TO  
  BSR   SAUT           ; lecture bloc (secteur de boot)
  BSR   SAUT           ; lecture bloc (secteur sdplay)
  BSR   SAUT           ; lecture bloc (secteur sdplay)
  BSR   SAUT           ; lecture bloc (secteur sdplay)
  BSR   SAUT           ; lecture bloc (secteur sdplay)
  BSR   SAUT           ; lecture bloc (secteur sdplay)
  BSR   SAUT           ; lecture bloc (secteur sdplay)
  BSR   SAUT           ; lecture bloc (secteur sdplay)
  BRA   VIDEO          ; jouer la video
    
;------------------------------------------------------
; Saut d'un bloc de la carte SD
;------------------------------------------------------
SAUT
  LDA   #$7F           ; initialiser A pour lecture
  READ_BYTE            ; lecture d'un octet
  CMPB  #$FE           ; comparaison avec $FE
  BNE   SAUT           ; attente debut secteur
  LDX   #$0200         ; initialisation compteur
SAUT1
  READ_BYTE            ; lecture d'un octet
  LEAX  -1,X           ; decrementation compteur
  BNE   SAUT1          ; nouvelle lecture
  READ_BYTE            ; lecture d'un octet
  READ_BYTE            ; lecture d'un octet
  RTS                  ; retour

;------------------------------------------------------
; FIN DU PROGRAMME
;------------------------------------------------------
FIN
; arret de la lecture
  LDU   #CMD12         ; adresse commande CMD12 
  JSR   $A028          ; EXCMD = execution commande CMD12
;                        FIN+6 modifie en $E028 pour TO  
; recommencer la lecture du fichier
  LEAS  1,S            ; retablir pointeur pile systeme
  LDX   #READ          ; adresse du debut du programme
  PSHS  X              ; empiler pour retour au debut
  RTS

;-----------------------------------------------------
; Affichage de la video
; 53*2+8+194*170+227 = 106+8+32980+227 = 33321 s/bloc
; 172 echantillons 5162 Hz
;-----------------------------------------------------
VIDEO
  LDU   #$0000         ; adresse memoire video
  LEAU  -1,U           ; initialiser a l'adresse precedente
  LDA   #$7F           ; initialisation pour lecture octet
VIDEO1
  READ_BYTE            ; lecture octet                   (48)
  CMPB  #$FE           ; test debut de bloc               (2)
  BNE   VIDEO1         ; attente debut de bloc            (3)
  LDB   #$AA           ; compteur de boucle 170           (2)
  PSHS  B              ; empilage compteur                (6)
VIDEO2   
  LDA   #$7F           ; initialisation lecture octet     (2)
  READ_BYTE            ; lecture de octet son            (48)
  STB   <$CD           ; envoi echantillon                (4)
  READ_BYTE            ; lecture octet saut              (48)
  BEQ   FIN            ; fin du fichier video             (3) 
  CLRA                 ; saut poids fort                  (2)
  LEAU  D,U            ; adresse courante dans cran      (8)
VIDEO3
  CMPU  #$1F40         ; test de fin ecran                (5)
  BLO   VIDEO4         ; lire et afficher un octet        (3)
  LEAU  -$1F40,U       ; retour au debut                  (8)
VIDEO4
  LDA   #$7F           ; initialisation lecture octet     (2)
  READ_BYTE            ; lecture octet forme             (48)  
  STB   ,U             ; affichage octet forme            (4)
  DEC   ,S             ; decrementation compteur          (6)
  BNE   VIDEO2         ; fin du bloc                      (3) 
VIDEO5
  PULS  B              ; depilage compteur                (6)
  LDA   #$7F           ; initialisation lecture octet     (2)
  READ_BYTE            ; lecture octet son               (48)
  STB   <$CD           ; envoi echantillon                (4) 
  READ_BYTE            ; lecture octet son               (48) 
  PSHS  B              ; empilage echantillon             (6)
  LDA   #$7F           ; initialisation lecture octet     (2)
  READ_BYTE            ; premier octet de CRC            (48) 
  READ_BYTE            ; deuxieme octet de CRC           (48)
  PULS  B              ; depilage dernier echantillon     (6)
  STB   <$CD           ; envoi echantillon                (4)
  LBRA  VIDEO1         ; bloc suivant                     (5)
  
  END
